part of picsou_simulation;

class Remuneration {

  Logger logger = new Logger('picsou.Remuneration');

  double remuneration;
  double calculBase;
  double charges;
  double brut;
  double netImposable;

  List<ChargeEurlComputed> chargesComputed = new List();

  Remuneration.computeFromCharges(double remuneration, double calculBase, List<ChargeEurlComputed> chargesComputed) {
    this.remuneration = remuneration;
    this.calculBase = calculBase;
    this.chargesComputed = chargesComputed;

    this.charges = 0.00;
    for(ChargeEurlComputed chargeComputed in this.chargesComputed) {
      this.charges += chargeComputed.chargeComputed;
    }

    this.brut = remuneration + this.charges;

    List<ChargeEurlComputed> chargesComputedNonDeductibles = chargesComputed.where((ChargeEurlComputed chargeEurlComputed) {
      if(chargeEurlComputed.charge is ChargeEurlCsgCrds) {
        ChargeEurlCsgCrds charge = chargeEurlComputed.charge;
        if(charge.isDeductible == false) {
          return true;
        }
      }
      return false;
    }).toList();

    double sumChargesIndeductible = chargesComputedNonDeductibles.map((charge) => charge.chargeComputed).reduce((charge1, charge2) => charge1 + charge2);
    netImposable = remuneration + sumChargesIndeductible;
  }

  Map toPublicJson() {
    Map map = new Map();
    map["net"] = roundNumber(remuneration);
    map["base"] = roundNumber(calculBase);
    map["totalCharges"] = roundNumber(charges);
    map["brut"] = roundNumber(brut);
    map["netImposable"] = roundNumber(netImposable);

    List chargesDetail = new List();
    for(ChargeEurlComputed charge in chargesComputed) {
      chargesDetail.add(charge.toPublicJson());
    }

    map["charges"] = chargesDetail;

    return map;
  }

  String toString() {
    return """
      \n
      [Remuneration]
      remuneration = $remuneration
      charges = $charges
      chargesComputed = $chargesComputed
      """;
  }

}
