part of picsou_simulation;


ServiceSimulationEurl serviceSimulationEurl = new ServiceSimulationEurl();

class ServiceSimulationEurl {

  Logger logger = new Logger('picsou.ServiceSimulationEurl');

  Map<double, double> findNetsByPercentage(double turnover, double runningCost) {

    Map<double, double> netByPercentage = new Map();

    for (var i = 0; i <= 100; i++) {
      SimulationEurl simulationEurl = computeSimulationPercentage(turnover, runningCost, i * 1.0);
      netByPercentage[i] = simulationEurl.net;
    }
    return netByPercentage;
  }

  SimulationEurl findBestSimulation(double turnover, double runningCost) {

    SimulationEurl bestSimulation = computeSimulationPercentage(turnover, runningCost, 0.00);
    for (var i = 1; i <= 100; i++) {
      SimulationEurl simulationEurl = computeSimulationPercentage(turnover, runningCost, i * 1.0);
      if(simulationEurl.net > bestSimulation.net) {
        bestSimulation = simulationEurl;
      }
    }
    return bestSimulation;
  }

  SimulationEurl computeSimulationPercentage(double turnover, double runningCost, double remunerationPercentage) {

    if(remunerationPercentage < 0) {
      remunerationPercentage = 0.00;
    }

    if(remunerationPercentage > 100) {
      remunerationPercentage = 100.00;
    }

    double maxRemuneration = turnover - runningCost;
    return serviceUtil.findByDichotomy(100, 0.00, maxRemuneration, remunerationPercentage, (newLimit) => computeSimulationEmpiric(turnover, runningCost, newLimit), (_) => true, (computed) => (computed.remuneration.brut / maxRemuneration) * 100);
  }

  SimulationEurl computeSimulationEmpiric(double turnover, double runningCost, double remuneration) {

    double maxDividend = turnover - runningCost - remuneration;
    return serviceUtil.findByDichotomy(100, 0.00, maxDividend, 0.00, (newLimit) => computeSimulation(turnover, runningCost, remuneration, newLimit), (computed) => (computed != null && computed.treso > 0.00), (computed) => computed.treso, proportional: false);
  }

  SimulationEurl computeSimulation(double turnover, double runningCost, double remuneration, double dividendBrut) {

    Remuneration remunerationEurl = serviceRemuneration.computeRemuneration(remuneration, remuneration + dividendBrut);

    double profit = turnover - runningCost - remunerationEurl.brut;

    double impotIs = serviceIrIs.computeIs(profit);
    Dividend dividend = serviceDividend.computeDividend(dividendBrut);
    double ir = serviceIrIs.computeIr(remunerationEurl.netImposable + dividend.netImposable);

    double treso = turnover - runningCost - remunerationEurl.brut - impotIs - dividendBrut;

    double net = remunerationEurl.remuneration + dividend.net - ir;

    return new SimulationEurl(turnover, runningCost, remunerationEurl, impotIs, dividend, ir, net, treso);
  }
}

