part of picsou_simulation;


ServiceSimulationSas serviceSimulationSas = new ServiceSimulationSas();

class ServiceSimulationSas {

  Logger logger = new Logger('picsou.ServiceSimulationSas');

  Map<double, double> findNetsByPercentage(double turnover, double runningCost) {

    Map<double, double> netByPercentage = new Map();

    for (var i = 0; i <= 100; i++) {
      SimulationSas simulationSas = computeSimulationBySalaryPercentage(turnover, runningCost, i * 1.0);
      netByPercentage[i] = simulationSas.net;
    }
    return netByPercentage;
  }

  SimulationSas findBestSimulation(double turnover, double runningCost) {

    SimulationSas bestSimulation = computeSimulationBySalaryPercentage(turnover, runningCost, 0.00);
    for (var i = 1; i <= 100; i++) {
      SimulationSas simulationSas = computeSimulationBySalaryPercentage(turnover, runningCost, i * 1.0);
      if(simulationSas.net > bestSimulation.net) {
        bestSimulation = simulationSas;
      }
    }
    return bestSimulation;
  }

  SimulationSas computeSimulationBySalaryPercentage(double turnover, double runningCost, double salaryPercentage) {

    if(salaryPercentage < 0) {
      salaryPercentage = 0.00;
    }

    if(salaryPercentage > 100) {
      salaryPercentage = 100.00;
    }

    double base = turnover - runningCost;
    double extraGrossTarget = (salaryPercentage * base) / 100;

    double grossSalary = serviceSalary.findGrossSalaryFromExtraGross(turnover, runningCost, extraGrossTarget);

    return serviceSimulationSas.computeSimulation(turnover, runningCost, grossSalary);
  }

  SimulationSas computeSimulation(double turnover, double runningCost, double grossSalary) {

    Salary salarySas = serviceSalary.computeSalary(grossSalary);

    double profit = turnover - runningCost - salarySas.extraGrossSalary;

    if(profit < 0) {
      throw 'Profit cannot be negative';
    }

    double impotIs = serviceIrIs.computeIs(profit);
    Dividend dividend = serviceDividend.computeDividend(profit - impotIs);
    double ir = serviceIrIs.computeIr(salarySas.netImposable + dividend.netImposable);

    double net = salarySas.netSalary + dividend.net - ir;

    return new SimulationSas(turnover, runningCost, salarySas, impotIs, dividend, ir, net);
  }
}

